const http = require('http')
const express = require('express')
const cors = require('cors')

const { pataka: env } = require('ahau-env')()
const { ApolloServer } = require('apollo-server-express') // TODO replace with express-graphql (copy the ssb-ahau setup)
const { buildFederatedSchema } = require('@apollo/federation')
const Main = require('@ssb-graphql/main')
const Profile = require('@ssb-graphql/profile')
const Invite = require('@ssb-graphql/invite')
const Stats = require('@ssb-graphql/stats')
const Pataka = require('@ssb-graphql/pataka')

module.exports = function graphqlServer (ssb, cb) {
  const PORT = env.graphql.port
  const app = express()
  app.options('*', cors())

  const main = Main(ssb, { type: 'pataka' })
  const profile = Profile(ssb)
  const invite = Invite(ssb)
  const stats = Stats(ssb)
  const pataka = Pataka(ssb, profile.gettersWithCache)

  main.loadContext((err, context) => {
    if (err) throw err

    const apollo = new ApolloServer({
      schema: buildFederatedSchema([
        main,
        profile,
        invite,
        pataka,
        stats
      ]),
      context,
      mockEntireSchema: false
    })
    apollo.applyMiddleware({ app })

    const httpServer = http.createServer(app)
    apollo.installSubscriptionHandlers(httpServer)

    httpServer.listen(PORT, (err) => {
      if (err) return cb(err)

      cb(null, { port: PORT, apollo })
    })

    ssb.close.hook((close, args) => {
      httpServer.close()
      close(...args)
    })
  })
}
